/*
 * com/ngon/mpeg/MPEGFrame.java
 * Victor Liu > mailto:victor@n-gon.com
 *
 * This code is based on GPL'd code in MPEG_Play by Joerg Anders
 * which may be found at:
 *
 *     http://rnvs.informatik.tu-chemnitz.de/~jan/MPEG/MPEG_Play.html
 *
 *
 * This program is free software; you can ristribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package com.ngon.mpeg;

public class MPEGFrame { 

    final static public int TYPE_I = 0x1;
    final static public int TYPE_P = 0x2;
    final static public int TYPE_B = 0x3;

    final static protected int RED         = 0xFF0000;
    final static protected int GRN         = 0x00FF00;
    final static protected int BLU         = 0x0000FF;
    
    final static protected int U_FAC      = 0x166EA;     // 1.402   * 2^16
    final static protected int V_FAC      = 0x1C5A2;     // 1.772   * 2^16
    final static protected int U_DIFF_FAC = 0x0B6D2;     // 0.71414 * 2^16
    final static protected int V_DIFF_FAC = 0x0581A;     // 0.34414 * 2^16
    
    protected int rgb[];  // The pixels in RGB
    protected MPEGFrameStats stats;

    MPEGFrame(int[][] pixels, int w, int h) {
        this(pixels[0], pixels[1], pixels[2], w, h);
    }
    
    MPEGFrame(int yPix[], int uPix[], int vPix[], int w, int h) {
        int r, g, b, u, v, u_g, v_g, y;

        int yIdx1 = 0, yIdx2 = w;
        int size = w * h;
        rgb = new int[size];
        
        size >>>= 2; 
        
        for (int i = 0; i < size; i++) {
            v = vPix[i] - 128;
            u = uPix[i] - 128; 

            u_g = u * U_DIFF_FAC;
            v_g = v * V_DIFF_FAC;
        
            v *= V_FAC;
            u *= U_FAC;

            for (int j = 0; j < 2; j++) {
                y = yPix[yIdx1] << 16;
                r = (y + u);
                b = (y + v) >> 16;
                g = (y - u_g - v_g) >> 8;

                r = (r > RED) ? RED : (r < 0) ? 0 : r & RED;
                g = (g > GRN) ? GRN : (g < 0) ? 0 : g & GRN;
                b = (b > BLU) ? BLU : (b < 0) ? 0 : b & BLU;

                rgb[yIdx1] =  (r | g | b);
    
                yIdx1++;
    
                y = yPix[yIdx2] << 16;
                r = (y + u);
                b = (y + v) >> 16;
                g = (y - u_g - v_g) >> 8;

                r = (r > RED) ? RED : (r < 0) ? 0 : r & RED;
                g = (g > GRN) ? GRN : (g < 0) ? 0 : g & GRN;
                b = (b > BLU) ? BLU : (b < 0) ? 0 : b & BLU;

                rgb[yIdx2] =  (r | g | b);
                yIdx2++;
                
                if (yIdx2 % w == 0) {
                    yIdx2 += w;
                    yIdx1 += w;
                }
            }
        }
    }
    
    public int[] getRGB() {
        return rgb;
    }
    
    public int getRGB(int index) {
        return rgb[index];
    }
    
    public int getRed(int index) {
        return (RED & rgb[index]) >> 16;
    }
    
    public int getGreen(int index) {
        return (GRN & rgb[index]) >> 8;
    }
    
    public int getBlue(int index) {
        return (BLU & rgb[index]);
    }
    
    public int getLuminescence(int index) {
        return (int) (0.299 * getRed(index) + 
                      0.587 * getGreen(index) + 
                      0.114 * getBlue(index));
    }
    
    public void setStats(MPEGFrameStats stats) {
        this.stats = stats;
    }
    
    public MPEGFrameStats getStats() {
        return stats;
    }
}
