/*
 * com/ngon/mpeg/MPEGFrameStats.java
 * Victor Liu > mailto:victor@n-gon.com
 *
 * This code is based on GPL'd code in MPEG_Play by Joerg Anders
 * which may be found at:
 *
 *     http://rnvs.informatik.tu-chemnitz.de/~jan/MPEG/MPEG_Play.html
 *
 *
 * This program is free software; you can ristribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package com.ngon.mpeg;

import java.awt.Dimension;
import java.awt.Point;

public class MPEGFrameStats { 

    // Flags for recording statistics
    final static public int DIMENSIONS     = 0x1;
    final static public int FRAME_TYPE     = 0x2;
    final static public int MOTION_VECTORS = 0x5; // Also needs DIMENSIONS
    
    int flags;
    
    public Dimension dimensions;
    public Dimension mbDimensions;
    public int frameType;
    public Point[] motionVectors;
    
    protected boolean hasMotionVectors;
    
    MPEGFrameStats(int flags) {
        this.flags = flags;
    }
    
    public boolean isRecordingStat(int flag) {
        return ((flags & flag) != 0);
    }
    
    public MPEGFrameStats copyStats() {
        try {
            MPEGFrameStats clone = (MPEGFrameStats) this.clone();
            return clone;
        } catch (CloneNotSupportedException e) {
            System.err.println(e);
        }
        return null;
    }
    
    public Object clone() throws CloneNotSupportedException {
        MPEGFrameStats clone = new MPEGFrameStats(flags);
        if (isRecordingStat(DIMENSIONS)) {
            clone.setDimensions(this.dimensions.width, 
                                this.dimensions.height);
        }
        if (isRecordingStat(FRAME_TYPE)) {
            clone.setFrameType(this.frameType);
        }
        if (isRecordingStat(MOTION_VECTORS)) {
            if (hasMotionVectors) {
                int p = 0;
                for (int j = 0; j < this.mbDimensions.height; j++) {
                    for (int i = 0; i < this.mbDimensions.width; i++ ) {
                        Point pt = this.motionVectors[p];
                        if (pt != null) {
                            clone.motionVectors[p] = 
                                (Point) this.motionVectors[p].clone();
                        }
                        p++;
                    }
                }
            } else {
                clone.motionVectors = null;
            }
        }
        return (Object) clone;
    }
    
    void setDimensions(int width, int height) {
        this.dimensions = new Dimension(width, height);
        if (isRecordingStat(MOTION_VECTORS) && this.motionVectors == null) {
            int mbWidth  = MPEGVideoDecoder.getMacroblockDimension(width);
            int mbHeight = MPEGVideoDecoder.getMacroblockDimension(height);
            this.motionVectors = new Point[mbWidth * mbHeight];
            this.mbDimensions = new Dimension(mbWidth, mbHeight);
        }
    }
    
    public Dimension getDimensions() {
        return this.dimensions;
    }
    
    public Dimension getMacroblockDimensions() {
        return this.mbDimensions;
    }
    
    void setFrameType(int frameType) {
        this.frameType = frameType;
    }
    
    public int getFrameType() {
        return frameType;
    }
    
    void setMotionVector(int mbRow, int mbCol, Point vec) {
        this.hasMotionVectors = true;
        this.motionVectors[mbRow * mbDimensions.width + mbCol] = vec;
    }
    
    public Point[] getMotionVectors() {
        return motionVectors;
    }
    
    public void endFrame() {
        this.hasMotionVectors = false;
    }
}
