/*
 * com/ngon/mpeg/MPEGMotionVector.java
 * Victor Liu > mailto:victor@n-gon.com
 *
 * This code is based on GPL'd code in MPEG_Play by Joerg Anders
 * which may be found at:
 *
 *     http://rnvs.informatik.tu-chemnitz.de/~jan/MPEG/MPEG_Play.html
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package com.ngon.mpeg;

import java.awt.Point;

class MPEGMotionVector {
    
    final static public int Y = MPEGVideoDecoder.Y;
    final static public int U = MPEGVideoDecoder.U;
    final static public int V = MPEGVideoDecoder.V;
    
    boolean fullPixelFlag;
    int rBits;
    int f;
    int motionRange;
    int motionMax;
    int motionMin;
    int[] motionRight;
    int[] motionDown;
    boolean[] doHalfStepRight;
    boolean[] doHalfStepDown;
    int right;
    int down;
    int prevRight;
    int prevDown;
    
    MPEGMotionVector() {
        clearPreviousMotion();
        motionRight = new int[3];
        motionDown  = new int[3];
        doHalfStepRight = new boolean[3];
        doHalfStepDown  = new boolean[3];
    }
    
    void clearPreviousMotion() {
        prevRight = prevDown = 0;
    }
    
    void setMotionParams(boolean fullPixelFlag, int rBits) {
        this.fullPixelFlag = fullPixelFlag;
        this.rBits = rBits;
        this.f = 1 << rBits;
        this.motionRange = this.f << 5;
        this.motionMax = (this.f << 4) - 1;
        this.motionMin = -(this.f << 4);
    }
    
    /*
     * Sets all the current motion values based on received motion params.
     */
    void decodeMotionVector(int horzCode, int vertCode, int horzR, int vertR) {
        
        right = getDisplacement(horzCode, horzR, prevRight);
        prevRight = right;
        if (fullPixelFlag) {
            right <<= 1;
        }
        
        down = getDisplacement(vertCode, vertR, prevDown);
        prevDown = down;
        if (fullPixelFlag) {
            down <<= 1;
        }
        
        motionRight[Y] = right >> 1;
        motionDown[Y]  = down  >> 1;
        doHalfStepRight[Y] = (right & 0x1) != 0;
        doHalfStepDown[Y]  = (down  & 0x1) != 0;
        
        motionRight[U] = right >> 2;
        motionDown[U]  = down  >> 2;
        doHalfStepRight[U] = (right & 0x2) != 0;
        doHalfStepDown[U]  = (down  & 0x2) != 0;
        
        motionRight[V] = right >> 2;
        motionDown[V]  = down  >> 2;
        doHalfStepRight[V] = (right & 0x2) != 0;
        doHalfStepDown[V]  = (down  & 0x2) != 0;
    }
    
    Point getMotionVector() {
        return new Point(this.right, this.down);
    }
    
    // The internal method getDisplacement() computes the 
    // difference of the actual motion vector with respect to 
    // the last motion vector. Refer to ISO 11172-2 to understand 
    // the coding of the motion displacement. 
    private int getDisplacement(int code, int r, int prevDisplacement) {
        int delta;
        int displacement;
    
        if (f == 1 || code == 0) {
            delta = code;
        } else {
            delta = 1 + f * (Math.abs(code) - 1);
            delta += r;
            if (code < 0) {
                delta = -delta;
            }
        }
        
        displacement = prevDisplacement + delta;
        
        if (displacement > motionMax) {
            displacement -= motionRange;
        } else if (displacement < motionMin) {
            displacement += motionRange;
        }
        
        return displacement;
    }
}
