/*
 * com/ngon/tap/EvolEvent.java
 * Victor Liu > mailto:victor@n-gon.com
 *
 * Copyright (C) 2004 Victor Liu
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package com.ngon.tap;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.StringTokenizer;
import java.util.Vector;

/**
 * class EvolEvent
 */
class EvolEvent extends Evol {
    
    Taps events = null;
    
    public EvolEvent(Connection dbc) {
        super(dbc);
    }
    
    public void init() {
        super.init();
        if (events == null) {
            events = new Taps();
        }
    }
    
    public boolean evol(Taps taps, int maxTaps, ClipSegFactory csFactory) {
        // Do something even if taps is empty.
        if (taps.size() == 0) {
            taps.addElement(null);
        }
        
        events.removeAllElements();
        for (int i = 0; i < taps.size(); i++) {
            Tap tap = (Tap) taps.elementAt(i);
            
            // If tap is null, record Add event.
            if (tap == null) {
                Clus c = selectInitialCluster();
                c.blocks = getBlocks(c);
                events.addElement(new TapEvent(c, TapEvent.EVENT_ADD));
                continue;
            }
            
            // If tap's clip has ended, record Drop event.
            ClipInfo ci = (ClipInfo) clipInfo.get(new Integer(tap.clus.clipID));
            if (ci == null) {
                continue;
            }
            if (tap.clus.frameNr > ci.clipLen - 2) {
                events.addElement(new TapEvent(null, TapEvent.EVENT_DROP));
                continue;
            }
            
            // Look for next cluster in sequence.  
            // If successful, record Change event.
            Clus next = queryInSequence.selectCluster(tap.clus);
            if (next == null) {
                // Copy current tap.
                next = new Clus();
                next.copyFields(tap.clus);
                next.blocks = tap.clus.blocks;
                next.frameNr++;
            } else {
                next.blocks = getBlocks(next);
            }
            events.addElement(new TapEvent(next, TapEvent.EVENT_CHANGE));
            
            // For last cluster in taps, chance to branch 
            // out of sequence.
            if (i == taps.size()-1 &&
                (int) (Math.random() * RANDOM) == 0) {
                Clus branch = queryOutOfSequence.selectCluster(next);
                if (branch == null) {
                    break;
                }
                branch.blocks = getBlocks(branch);
                events.addElement(new TapEvent(branch, TapEvent.EVENT_ADD));
            }
        }
        
        // This shouldn't normally happen.
        if (events.size() > maxTaps) {
            // Remove first already-existing tap.
            for (int i = 0; i < events.size(); i++) {
                TapEvent evt = (TapEvent) events.elementAt(i);
                if (evt.actionCode.compareTo(TapEvent.EVENT_CHANGE) == 0) {
                    events.removeElementAt(i);
                    events.insertElementAt(
                        new TapEvent(null, TapEvent.EVENT_DROP), i);
                    break;
                }
            }
        }
        
        // Update taps to be events pruned of actions.
        taps.removeAllElements();
        for (Enumeration e = events.elements(); e.hasMoreElements(); ) {
            TapEvent evt = (TapEvent) e.nextElement();
            if (evt.actionCode.compareTo(TapEvent.EVENT_DROP) == 0) {
                continue;
            }
            taps.addElement((Tap) evt);
        }
        
        return true;
    }
    
    public void kill() {
        if (events != null) {
            events.removeAllElements();
            events = null;
        }
    }
    
    public Taps getEvents() {
        return events;
    }
    
    /*
     * Clean up resources.  Necessary to release db connection back to pool.
     */
    public void releaseDB() {
        try {
            queryInSequence.close();
            queryInSequence = null;
            queryOutOfSequence.close();
            queryOutOfSequence = null;
            clipQuery.close();
            clipQuery = null;
            blocksQuery.close();
            blocksQuery = null;
            clusQueryByID.close();
            clusQueryByID = null;
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            if (queryInSequence != null) {
                queryInSequence.close();
                queryInSequence = null;
            }
            if (queryOutOfSequence != null) {
                queryOutOfSequence.close();
                queryOutOfSequence = null;
            }
            if (clipQuery != null) {
                try { clipQuery.close(); } catch (SQLException e) {}
                clipQuery = null;
            }
            if (blocksQuery != null) {
                try { blocksQuery.close(); } catch (SQLException e) {}
                blocksQuery = null;
            }
            if (clusQueryByID != null) {
                try { clusQueryByID.close(); } catch (SQLException e) {}
                clusQueryByID = null;
            }
        }
    }
    
    /*
     * Same as super's except for creating clip.
     */
    protected void addNewTap(Taps taps, ClipSegFactory csFactory) {
        Clus c = selectInitialCluster();
        // Get blocks of cluster.
        c.blocks = getBlocks(c);
        taps.addElement(new Tap(c));
    }
}

